@@ -4,13 +4,20 @@
 
 //! Skeletons are used for pattern matching. See the [`Skeleton`] struct for more information.
 
+#[cfg(feature = "provider_serde")]
+use alloc::format;
+#[cfg(feature = "provider_serde")]
+use alloc::string::String;
+
+use alloc::vec::Vec;
+use core::convert::TryFrom;
+use displaydoc::Display;
 use smallvec::SmallVec;
-use std::{convert::TryFrom, fmt};
 
 use crate::{
     fields::{self, Field, FieldLength, FieldSymbol},
-    options::length,
-    pattern::Pattern,
+    options::{length, preferences},
+    pattern::{Pattern, PatternItem},
     provider::gregory::patterns::{LengthPatternsV1, PatternV1, SkeletonV1, SkeletonsV1},
 };
 
@@ -47,6 +54,11 @@ impl Skeleton {
     fn fields_len(&self) -> usize {
         self.0.len()
     }
+
+    /// Return the underlying fields as a slice.
+    pub fn as_slice(&self) -> &[fields::Field] {
+        self.0.as_slice()
+    }
 }
 
 /// This is an implementation of the serde deserialization visitor pattern.
@@ -58,7 +70,7 @@ struct DeserializeSkeletonFieldsUTS35String;
 impl<'de> de::Visitor<'de> for DeserializeSkeletonFieldsUTS35String {
     type Value = Skeleton;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut core::fmt::Formatter) -> core::fmt::Result {
         write!(formatter, "Expected to find a valid skeleton.")
     }
 
@@ -87,7 +99,7 @@ struct DeserializeSkeletonBincode;
 impl<'de> de::Visitor<'de> for DeserializeSkeletonBincode {
     type Value = Skeleton;
 
-    fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+    fn expecting(&self, formatter: &mut core::fmt::Formatter) -> core::fmt::Result {
         write!(formatter, "Unable to deserialize a bincode Pattern.")
     }
 
@@ -194,6 +206,65 @@ impl TryFrom<&str> for Skeleton {
     }
 }
 
+/// Convert a Pattern into a Skeleton. This will remove all of the string literals, and sort
+/// the fields into the canonical sort order. Not all fields are supported by Skeletons, so map
+/// fields into skeleton-appropriate ones. For instance, in the "ja" locale the pattern "aK:mm"
+/// gets transformed into the skeleton "hmm".
+///
+/// At the time of this writing, it's being used for applying hour cycle preferences and should not
+/// be exposed as a public API for end users.
+#[doc(hidden)]
+#[cfg(feature = "provider_transform_internals")]
+impl From<&Pattern> for Skeleton {
+    fn from(pattern: &Pattern) -> Self {
+        let mut fields: SmallVec<[fields::Field; 5]> = SmallVec::new();
+        for item in pattern.items() {
+            if let crate::pattern::PatternItem::Field(field) = item {
+                let mut field = *field;
+
+                // Skeletons only have a subset of available fields, these are then mapped to more
+                // specific fields for the patterns they expand to.
+                field.symbol = match field.symbol {
+                    // Only the format varieties are used in the skeletons, the matched patterns
+                    // will be more specific.
+                    FieldSymbol::Month(_) => FieldSymbol::Month(fields::Month::Format),
+                    FieldSymbol::Weekday(_) => FieldSymbol::Weekday(fields::Weekday::Format),
+
+                    // Only flexible day periods are used in skeletons, ignore all others.
+                    FieldSymbol::DayPeriod(fields::DayPeriod::AmPm)
+                    | FieldSymbol::DayPeriod(fields::DayPeriod::NoonMidnight) => continue,
+                    // TODO(#487) - Flexible day periods should be included here.
+                    // FieldSymbol::DayPeriod(fields::DayPeriod::Flexible) => {
+                    //     FieldSymbol::DayPeriod(fields::DayPeriod::Flexible)
+                    // }
+
+                    // Only the H12 and H23 symbols are used in skeletons, while the patterns may
+                    // contain H11 or H23 depending on the localization.
+                    FieldSymbol::Hour(fields::Hour::H11) | FieldSymbol::Hour(fields::Hour::H12) => {
+                        FieldSymbol::Hour(fields::Hour::H12)
+                    }
+                    FieldSymbol::Hour(fields::Hour::H23) | FieldSymbol::Hour(fields::Hour::H24) => {
+                        FieldSymbol::Hour(fields::Hour::H23)
+                    }
+
+                    // Pass through all of the following preferences unchanged.
+                    FieldSymbol::Minute
+                    | FieldSymbol::Second(_)
+                    | FieldSymbol::TimeZone(_)
+                    | FieldSymbol::Year(_)
+                    | FieldSymbol::Day(_) => field.symbol,
+                };
+
+                // Only insert if it's a unique field.
+                if let Err(pos) = fields.binary_search(&field) {
+                    fields.insert(pos, field)
+                }
+            }
+        }
+        Self(fields)
+    }
+}
+
 /// Represents a specific pattern that is available for a given locale.
 /// A [`Skeleton`] is used to match against to find the best pattern.
 #[derive(Debug, PartialEq, Clone)]
@@ -214,43 +285,35 @@ impl<'a> From<(&'a SkeletonV1, &'a PatternV1)> for AvailableFormatPattern<'a> {
     }
 }
 
-#[derive(Debug)]
-pub enum SkeletonError {
-    InvalidFieldLength,
-    DuplicateField,
-    SymbolUnknown(char),
-    SymbolInvalid(char),
-    SymbolUnimplemented(char),
-    UnimplementedField(char),
-    Fields(fields::Error),
-}
-
 /// These strings follow the recommendations for the serde::de::Unexpected::Other type.
 /// https://docs.serde.rs/serde/de/enum.Unexpected.html#variant.Other
 ///
 /// Serde will generate an error such as:
 /// "invalid value: unclosed literal in pattern, expected a valid UTS 35 pattern string at line 1 column 12"
-impl fmt::Display for SkeletonError {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        match self {
-            Self::InvalidFieldLength => write!(f, "field too long in skeleton"),
-            Self::DuplicateField => write!(f, "duplicate field in skeleton"),
-            Self::SymbolUnknown(ch) => write!(f, "symbol unknown {} in skeleton", ch),
-            Self::SymbolInvalid(ch) => write!(f, "symbol invalid {} in skeleton", ch),
-            Self::SymbolUnimplemented(ch) => {
-                write!(f, "symbol unimplemented {} in skeleton", ch)
-            }
-            Self::UnimplementedField(ch) => {
-                write!(f, "unimplemented field {} in skeleton", ch)
-            }
-            Self::Fields(err) => write!(f, "{} in skeleton", err),
-        }
-    }
+#[derive(Display, Debug)]
+pub enum SkeletonError {
+    #[displaydoc("field too long in skeleton")]
+    InvalidFieldLength,
+    #[displaydoc("duplicate field in skeleton")]
+    DuplicateField,
+    #[displaydoc("symbol unknown {0} in skeleton")]
+    SymbolUnknown(char),
+    #[displaydoc("symbol invalid {0} in skeleton")]
+    SymbolInvalid(char),
+    #[displaydoc("symbol unimplemented {0} in skeleton")]
+    SymbolUnimplemented(char),
+    #[displaydoc("unimplemented field {0} in skeleton")]
+    UnimplementedField(char),
+    #[displaydoc("{0}")]
+    Fields(fields::Error),
 }
 
+#[cfg(feature = "std")]
+impl std::error::Error for SkeletonError {}
+
 impl From<fields::Error> for SkeletonError {
-    fn from(fields_error: fields::Error) -> Self {
-        Self::Fields(fields_error)
+    fn from(e: fields::Error) -> Self {
+        SkeletonError::Fields(e)
     }
 }
 
@@ -267,7 +330,7 @@ impl From<fields::SymbolError> for SkeletonError {
             fields::SymbolError::Unknown(byte) => {
                 // NOTE: If you remove a symbol due to it now being supported,
                 //       make sure to regenerate the test data.
-                //       https://github.com/unicode-org/icu4x/blob/main/resources/testdata/README.md
+                //       https://github.com/unicode-org/icu4x/blob/main/provider/testdata/README.md
                 match byte {
                     // TODO(#487) - Flexible day periods
                     b'B'
@@ -357,20 +420,57 @@ pub enum BestSkeleton<T> {
     NoMatch,
 }
 
+/// The hour cycle can be set by preferences. This function switches between h11 and h12,
+/// and between h23 and h24. This function is naive as it is assumed that this application of
+/// the hour cycle will not change between h1x to h2x.
+fn naively_apply_hour_cycle_preferences(
+    pattern: &mut Pattern,
+    preferences: &Option<preferences::Bag>,
+) {
+    // If there is a preference overiding the hour cycle, apply it now.
+    if let Some(preferences::Bag {
+        hour_cycle: Some(hour_cycle),
+    }) = preferences
+    {
+        for item in pattern.items_mut() {
+            if let PatternItem::Field(fields::Field { symbol, length: _ }) = item {
+                if let fields::FieldSymbol::Hour(_) = symbol {
+                    *symbol = fields::FieldSymbol::Hour(hour_cycle.field());
+                }
+            }
+        }
+    }
+}
+
 // TODO - This could return a Cow<'a, Pattern>, but it affects every other part of the API to
 // add a lifetime here. The pattern returned here could be one that we've already constructed in
 // the CLDR as an exotic type, or it could be one that was modified to meet the requirements of
 // the components bag.
+
+/// Given a set of fields (which represents a skeleton), try to create a best localized pattern
+// for those fields.
+///
+/// * `skeletons` - The skeletons that will be matched against
+/// * `length_patterns` - Contains information on how to combine date and time patterns.
+/// * `fields` - The desired fields to match against.
+/// * `prefer_matched_pattern` - This algorithm does some extra steps of trying to respect
+///         the desired fields, even if the provider data doesn't completely match. This
+///         configuration option makes it so that the final pattern won't have additional work
+///         done to mutate it to match the fields. It will prefer the actual matched pattern.
 pub fn create_best_pattern_for_fields<'a>(
     skeletons: &'a SkeletonsV1,
     length_patterns: &LengthPatternsV1,
     fields: &[Field],
+    preferences: &Option<preferences::Bag>,
+    prefer_matched_pattern: bool,
 ) -> BestSkeleton<Pattern> {
-    let first_pattern_match = get_best_available_format_pattern(skeletons, fields);
+    let first_pattern_match =
+        get_best_available_format_pattern(skeletons, fields, prefer_matched_pattern);
 
     // Try to match a skeleton to all of the fields.
-    if let BestSkeleton::AllFieldsMatch(pattern) = first_pattern_match {
-        return BestSkeleton::AllFieldsMatch(pattern.clone());
+    if let BestSkeleton::AllFieldsMatch(mut pattern) = first_pattern_match {
+        naively_apply_hour_cycle_preferences(&mut pattern, &preferences);
+        return BestSkeleton::AllFieldsMatch(pattern);
     }
 
     let FieldsByType { date, time, other } = group_fields_by_type(fields);
@@ -380,9 +480,14 @@ pub fn create_best_pattern_for_fields<'a>(
         // TODO(#583) - TimeZones
         // TODO(#486) - Eras,
         // ... etc.
-        unimplemented!(
-            "There are no \"other\" fields supported, these need to be appended to the pattern. {:?}", other
-        );
+
+        // TODO(#583) - This is commented out because TimeZone support is required here in order to
+        // generate length::Bag patterns correctly. For now it's commented out so that everything
+        // works for length::Bag with a preference, although it may lack a time zone.
+
+        // unimplemented!(
+        //     "There are no \"other\" fields supported, these need to be appended to the pattern. {:?}", other
+        // );
     }
 
     if date.is_empty() || time.is_empty() {
@@ -390,8 +495,11 @@ pub fn create_best_pattern_for_fields<'a>(
             BestSkeleton::AllFieldsMatch(_) => {
                 unreachable!("Logic error in implementation. AllFieldsMatch handled above.")
             }
-            BestSkeleton::MissingOrExtraFields(pattern) => {
-                BestSkeleton::MissingOrExtraFields(pattern.clone())
+            BestSkeleton::MissingOrExtraFields(mut pattern) => {
+                if date.is_empty() {
+                    naively_apply_hour_cycle_preferences(&mut pattern, &preferences);
+                }
+                BestSkeleton::MissingOrExtraFields(pattern)
             }
             BestSkeleton::NoMatch => BestSkeleton::NoMatch,
         };
@@ -400,19 +508,23 @@ pub fn create_best_pattern_for_fields<'a>(
     // Match the date and time, and then simplify the combinatorial logic of the results into
     // an optional values of the results, and a boolean value.
     let (date_pattern, date_missing_or_extra) =
-        match get_best_available_format_pattern(skeletons, &date) {
+        match get_best_available_format_pattern(skeletons, &date, prefer_matched_pattern) {
             BestSkeleton::MissingOrExtraFields(fields) => (Some(fields), true),
             BestSkeleton::AllFieldsMatch(fields) => (Some(fields), false),
             BestSkeleton::NoMatch => (None, true),
         };
 
-    let (time_pattern, time_missing_or_extra) =
-        match get_best_available_format_pattern(skeletons, &time) {
+    let (mut time_pattern, time_missing_or_extra) =
+        match get_best_available_format_pattern(skeletons, &time, prefer_matched_pattern) {
             BestSkeleton::MissingOrExtraFields(fields) => (Some(fields), true),
             BestSkeleton::AllFieldsMatch(fields) => (Some(fields), false),
             BestSkeleton::NoMatch => (None, true),
         };
 
+    if let Some(ref mut pattern) = time_pattern {
+        naively_apply_hour_cycle_preferences(pattern, &preferences)
+    }
+
     // Determine how to combine the date and time.
     let pattern: Option<Pattern> = match (date_pattern, time_pattern) {
         (Some(date_pattern), Some(time_pattern)) => {
@@ -453,13 +565,10 @@ pub fn create_best_pattern_for_fields<'a>(
                 length::Date::Short => &length_patterns.short,
             };
 
-            Some(
-                Pattern::from_bytes_combination(bytes, date_pattern.clone(), time_pattern.clone())
-                    .expect("TODO"),
-            )
+            Some(Pattern::from_bytes_combination(bytes, date_pattern, time_pattern).expect("TODO"))
         }
-        (None, Some(pattern)) => Some(pattern.clone()),
-        (Some(pattern), None) => Some(pattern.clone()),
+        (None, Some(pattern)) => Some(pattern),
+        (Some(pattern), None) => Some(pattern),
         (None, None) => None,
     };
 
@@ -521,22 +630,21 @@ fn group_fields_by_type(fields: &[Field]) -> FieldsByType {
 ///
 ///  * Compute a score based on the best possible match for the given fields.
 ///  * Select the skeleton with highest score.
-///
-/// The following is not implemented:
-///
-///  * 2.6.2.1 Matching Skeletons
-///    - TODO(#584) - Modify the resulting pattern to have fields of the same length. For example requesting
+///  * Modify the resulting pattern to have fields of the same length. For example requesting
 ///      a skeleton "yMMMMd" can have a best match of ["yMMMd", "d MMM y"]. This pattern should
 ///      then be modified to use the requested length to produce a pattern "d MMMM y".
 ///      However, fields should not be changed from numeric to text.
+///
+/// The following is not implemented:
+///
 ///  * 2.6.2.2 Missing Skeleton Fields
-///    - TODO(#585) - The mechanism to combine a date pattern and a time pattern.
 ///    - TODO(#586) - Using the CLDR appendItems field. Note: There is not agreement yet on how
 ///      much of this step to implement. See the issue for more information.
-pub fn get_best_available_format_pattern<'a>(
-    skeletons: &'a SkeletonsV1,
+pub fn get_best_available_format_pattern(
+    skeletons: &SkeletonsV1,
     fields: &[Field],
-) -> BestSkeleton<&'a Pattern> {
+    prefer_matched_pattern: bool,
+) -> BestSkeleton<Pattern> {
     let mut closest_format_pattern = None;
     let mut closest_distance: u32 = u32::MAX;
     let mut closest_missing_fields = 0;
@@ -626,10 +734,49 @@ pub fn get_best_available_format_pattern<'a>(
     if closest_missing_fields == fields.len() {
         return BestSkeleton::NoMatch;
     }
-    if closest_distance >= SKELETON_EXTRA_SYMBOL {
-        return BestSkeleton::MissingOrExtraFields(closest_format_pattern);
+
+    if closest_distance == NO_DISTANCE {
+        return BestSkeleton::AllFieldsMatch(closest_format_pattern.clone());
     }
-    BestSkeleton::AllFieldsMatch(closest_format_pattern)
+
+    // Modify the resulting pattern to have fields of the same length.
+    let expanded_pattern = if prefer_matched_pattern {
+        #[cfg(not(feature = "provider_transform_internals"))]
+        panic!("This code branch should only be run when transforming provider code.");
+
+        #[cfg(feature = "provider_transform_internals")]
+        closest_format_pattern.clone()
+    } else {
+        Pattern::from(
+            closest_format_pattern
+                .items()
+                .iter()
+                .map(|item| {
+                    if let PatternItem::Field(pattern_field) = item {
+                        if let Some(requested_field) = fields
+                            .iter()
+                            .find(|field| field.symbol == pattern_field.symbol)
+                        {
+                            if requested_field.length != pattern_field.length
+                                && requested_field.get_length_type()
+                                    == pattern_field.get_length_type()
+                            {
+                                return PatternItem::Field(*requested_field);
+                            }
+                        }
+                    }
+                    // There's no match, or this is a string literal return the original item.
+                    item.clone()
+                })
+                .collect::<Vec<PatternItem>>(),
+        )
+    };
+
+    if closest_distance >= SKELETON_EXTRA_SYMBOL {
+        return BestSkeleton::MissingOrExtraFields(expanded_pattern);
+    }
+
+    BestSkeleton::AllFieldsMatch(expanded_pattern)
 }
 
 pub fn get_available_format_patterns<'a>(
@@ -643,22 +790,21 @@ mod test {
     use super::*;
 
     use icu_locid_macros::langid;
-    use icu_provider::{DataProvider, DataRequest, ResourceOptions, ResourcePath};
-    use std::borrow::Cow;
+    use icu_provider::prelude::*;
 
     use crate::{
         fields::{Day, Field, FieldLength, Month, Weekday},
         options::components,
-        provider::{gregory::DatesV1, key::GREGORY_V1},
+        provider::{gregory::DatePatternsV1Marker, key::GREGORY_DATE_PATTERNS_V1},
     };
 
-    fn get_data_provider() -> Cow<'static, DatesV1> {
+    fn get_data_payload() -> DataPayload<'static, DatePatternsV1Marker> {
         let provider = icu_testdata::get_provider();
         let langid = langid!("en");
         provider
             .load_payload(&DataRequest {
                 resource_path: ResourcePath {
-                    key: GREGORY_V1,
+                    key: GREGORY_DATE_PATTERNS_V1,
                     options: ResourceOptions {
                         variant: None,
                         langid: Some(langid),
@@ -666,8 +812,7 @@ mod test {
                 },
             })
             .unwrap()
-            .payload
-            .take()
+            .take_payload()
             .unwrap()
     }
 
@@ -687,17 +832,18 @@ mod test {
             ..Default::default()
         };
         let requested_fields = components.to_vec_fields();
-        let data_provider = get_data_provider();
+        let data_provider = get_data_payload();
 
         match get_best_available_format_pattern(
-            &data_provider.patterns.datetime.skeletons,
+            &data_provider.get().datetime.skeletons,
             &requested_fields,
+            false,
         ) {
             BestSkeleton::AllFieldsMatch(available_format_pattern)
             | BestSkeleton::MissingOrExtraFields(available_format_pattern) => {
                 assert_eq!(
                     available_format_pattern.to_string(),
-                    String::from("MMM d, y")
+                    String::from("MMMM d, y")
                 )
             }
             BestSkeleton::NoMatch => {
@@ -714,11 +860,12 @@ mod test {
             ..Default::default()
         };
         let requested_fields = components.to_vec_fields();
-        let data_provider = get_data_provider();
+        let data_provider = get_data_payload();
 
         match get_best_available_format_pattern(
-            &data_provider.patterns.datetime.skeletons,
+            &data_provider.get().datetime.skeletons,
             &requested_fields,
+            false,
         ) {
             BestSkeleton::MissingOrExtraFields(available_format_pattern) => {
                 assert_eq!(available_format_pattern.to_string(), String::from("L"))
@@ -739,19 +886,21 @@ mod test {
             ..Default::default()
         };
         let requested_fields = components.to_vec_fields();
-        let data_provider = get_data_provider();
+        let data_provider = get_data_payload();
 
         match create_best_pattern_for_fields(
-            &data_provider.patterns.datetime.skeletons,
-            &data_provider.patterns.datetime.length_patterns,
+            &data_provider.get().datetime.skeletons,
+            &data_provider.get().datetime.length_patterns,
             &requested_fields,
+            &None,
+            false,
         ) {
             BestSkeleton::AllFieldsMatch(available_format_pattern) => {
                 // TODO - This needs to support the "Z" pattern. This test will begin to fail
                 // once support is added.
                 assert_eq!(
                     available_format_pattern.to_string(),
-                    String::from("MMM d, y")
+                    String::from("MMMM d, y")
                 )
             }
             best => panic!("Unexpected {:?}", best),
@@ -762,12 +911,13 @@ mod test {
     fn test_skeleton_empty_bag() {
         let components: components::Bag = Default::default();
         let requested_fields = components.to_vec_fields();
-        let data_provider = get_data_provider();
+        let data_provider = get_data_payload();
 
         assert_eq!(
             get_best_available_format_pattern(
-                &data_provider.patterns.datetime.skeletons,
-                &requested_fields
+                &data_provider.get().datetime.skeletons,
+                &requested_fields,
+                false
             ),
             BestSkeleton::NoMatch,
             "No match was found"
@@ -783,12 +933,13 @@ mod test {
             ..Default::default()
         };
         let requested_fields = components.to_vec_fields();
-        let data_provider = get_data_provider();
+        let data_provider = get_data_payload();
 
         assert_eq!(
             get_best_available_format_pattern(
-                &data_provider.patterns.datetime.skeletons,
-                &requested_fields
+                &data_provider.get().datetime.skeletons,
+                &requested_fields,
+                false
             ),
             BestSkeleton::NoMatch,
             "No match was found"
@@ -813,7 +964,7 @@ mod test {
     // NOTE: If you are moving this to the SUPPORTED section, make sure to remove the match
     //       on your symbol from impl From<fields::SymbolError> for SkeletonError
     //       and then regenerate the test data.
-    //       https://github.com/unicode-org/icu4x/blob/main/resources/testdata/README.md
+    //       https://github.com/unicode-org/icu4x/blob/main/provider/testdata/README.md
     #[rustfmt::skip]
     const UNSUPPORTED_STRING_SKELETONS: [&str; 19] = [
         // TODO(#487) - Flexible day periods
@@ -901,7 +1052,7 @@ mod test {
             .collect();
 
         for (strings, fields) in skeletons_strings.windows(2).zip(skeleton_fields.windows(2)) {
-            if fields[0].cmp(&fields[1]) != std::cmp::Ordering::Less {
+            if fields[0].cmp(&fields[1]) != core::cmp::Ordering::Less {
                 panic!("Expected {:?} < {:?}", strings[0], strings[1]);
             }
         }
@@ -999,4 +1150,51 @@ mod test {
             "invalid value: field item out of order or duplicate: Field { symbol: Day(DayOfMonth), length: One }, expected ordered field symbols representing a skeleton"
         );
     }
+
+    #[cfg(feature = "provider_transform_internals")]
+    fn assert_pattern_to_skeleton(pattern: &str, skeleton: &str, message: &str) {
+        assert_eq!(
+            serde_json::to_string(skeleton).expect("Failed to transform skeleton to string."),
+            serde_json::to_string(&Skeleton::from(
+                &Pattern::from_bytes(pattern).expect("Failed to create pattern from bytes.")
+            ))
+            .expect("Failed to transform skeleton to string."),
+            "{}",
+            message
+        );
+    }
+
+    #[test]
+    #[cfg(feature = "provider_transform_internals")]
+    fn test_pattern_to_skeleton() {
+        assert_pattern_to_skeleton("H:mm:ss v", "Hmmssv", "Test a complicated time pattern");
+        assert_pattern_to_skeleton(
+            "v ss:mm:H",
+            "Hmmssv",
+            "Test the skeleton ordering is consistent",
+        );
+
+        assert_pattern_to_skeleton("K:mm", "hmm", "H11 maps to H12");
+        assert_pattern_to_skeleton("k:mm", "Hmm", "H23 maps to H24");
+
+        assert_pattern_to_skeleton("ha mm", "hmm", "Day periods get removed");
+        assert_pattern_to_skeleton("h 'at' b mm", "hmm", "Day periods get removed");
+
+        assert_pattern_to_skeleton("y", "y", "The year is passed through");
+        assert_pattern_to_skeleton("Y", "Y", "The year is passed through");
+
+        assert_pattern_to_skeleton("LLL", "MMM", "Remove standalone months.");
+
+        assert_pattern_to_skeleton("s", "s", "Seconds pass through");
+        assert_pattern_to_skeleton("S", "S", "Seconds pass through");
+        assert_pattern_to_skeleton("A", "A", "Seconds pass through");
+
+        assert_pattern_to_skeleton("z", "z", "Timezones get passed through");
+        assert_pattern_to_skeleton("Z", "Z", "Timezones get passed through");
+        assert_pattern_to_skeleton("O", "O", "Timezones get passed through");
+        assert_pattern_to_skeleton("v", "v", "Timezones get passed through");
+        assert_pattern_to_skeleton("V", "V", "Timezones get passed through");
+        assert_pattern_to_skeleton("X", "X", "Timezones get passed through");
+        assert_pattern_to_skeleton("x", "x", "Timezones get passed through");
+    }
 }
