@@ -2,30 +2,33 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-use std::{borrow::Cow, fmt};
+use alloc::borrow::Cow;
+use alloc::format;
+use alloc::string::String;
+use core::fmt;
 
 use crate::{
     date::TimeZoneInput, format::time_zone::FormattedTimeZone, pattern::Error as PatternError,
     provider, DateTimeFormatError,
 };
-use crate::{format::time_zone, provider::time_zones::TimeZoneFormatsV1};
+use crate::{format::time_zone, provider::time_zones::TimeZoneFormatsV1Marker};
 use icu_locid::{LanguageIdentifier, Locale};
-use icu_provider::{DataProvider, DataRequest, ResourceKey, ResourceOptions, ResourcePath};
+use icu_provider::prelude::*;
 
 use crate::fields::{FieldSymbol, TimeZone};
 use crate::pattern::{Pattern, PatternItem};
 
 /// Loads a resource into its destination if the destination has not already been filled.
-fn load_resource<'d, D, L, P>(
+fn load_resource<'data, D, L, P>(
     locale: &L,
     resource_key: ResourceKey,
-    destination: &mut Option<Cow<'d, D>>,
+    destination: &mut Option<DataPayload<'data, D>>,
     provider: &P,
 ) -> Result<(), DateTimeFormatError>
 where
-    D: std::fmt::Debug + Clone + 'd,
+    D: DataMarker<'data>,
     L: Clone + Into<LanguageIdentifier>,
-    P: DataProvider<'d, D> + ?Sized,
+    P: DataProvider<'data, D> + ?Sized,
 {
     if destination.is_none() {
         *destination = Some(
@@ -39,8 +42,7 @@ where
                         },
                     },
                 })?
-                .payload
-                .take()?,
+                .take_payload()?,
         );
     }
     Ok(())
@@ -85,28 +87,29 @@ where
 /// // let value = tzf.format_to_string(&time_zone);
 /// // ```
 // TODO(#622) Make TimeZoneFormat public once we have a clean way to provide it options.
-pub(super) struct TimeZoneFormat<'d> {
+pub(super) struct TimeZoneFormat<'data> {
     /// The pattern to format.
     pub(super) pattern: Pattern,
     /// The data that contains meta information about how to display content.
-    pub(super) zone_formats: Cow<'d, provider::time_zones::TimeZoneFormatsV1<'d>>,
+    pub(super) zone_formats: DataPayload<'data, provider::time_zones::TimeZoneFormatsV1Marker>,
     /// The exemplar cities for time zones.
-    pub(super) exemplar_cities: Option<Cow<'d, provider::time_zones::ExemplarCitiesV1<'d>>>,
+    pub(super) exemplar_cities:
+        Option<DataPayload<'data, provider::time_zones::ExemplarCitiesV1Marker>>,
     /// The generic long metazone names, e.g. Pacific Time
     pub(super) mz_generic_long:
-        Option<Cow<'d, provider::time_zones::MetaZoneGenericNamesLongV1<'d>>>,
+        Option<DataPayload<'data, provider::time_zones::MetaZoneGenericNamesLongV1Marker>>,
     /// The generic short metazone names, e.g. PT
     pub(super) mz_generic_short:
-        Option<Cow<'d, provider::time_zones::MetaZoneGenericNamesShortV1<'d>>>,
+        Option<DataPayload<'data, provider::time_zones::MetaZoneGenericNamesShortV1Marker>>,
     /// The specific long metazone names, e.g. Pacific Daylight Time
     pub(super) mz_specific_long:
-        Option<Cow<'d, provider::time_zones::MetaZoneSpecificNamesLongV1<'d>>>,
+        Option<DataPayload<'data, provider::time_zones::MetaZoneSpecificNamesLongV1Marker>>,
     /// The specific short metazone names, e.g. Pacific Daylight Time
     pub(super) mz_specific_short:
-        Option<Cow<'d, provider::time_zones::MetaZoneSpecificNamesShortV1<'d>>>,
+        Option<DataPayload<'data, provider::time_zones::MetaZoneSpecificNamesShortV1Marker>>,
 }
 
-impl<'d> TimeZoneFormat<'d> {
+impl<'data> TimeZoneFormat<'data> {
     /// Constructor that selectively loads data based on what is required to
     /// format the given pattern into the given locale.
     ///
@@ -136,17 +139,17 @@ impl<'d> TimeZoneFormat<'d> {
     ) -> Result<Self, DateTimeFormatError>
     where
         L: Into<Locale>,
-        ZP: DataProvider<'d, provider::time_zones::TimeZoneFormatsV1<'d>>
-            + DataProvider<'d, provider::time_zones::ExemplarCitiesV1<'d>>
-            + DataProvider<'d, provider::time_zones::MetaZoneGenericNamesLongV1<'d>>
-            + DataProvider<'d, provider::time_zones::MetaZoneGenericNamesShortV1<'d>>
-            + DataProvider<'d, provider::time_zones::MetaZoneSpecificNamesLongV1<'d>>
-            + DataProvider<'d, provider::time_zones::MetaZoneSpecificNamesShortV1<'d>>
+        ZP: DataProvider<'data, provider::time_zones::TimeZoneFormatsV1Marker>
+            + DataProvider<'data, provider::time_zones::ExemplarCitiesV1Marker>
+            + DataProvider<'data, provider::time_zones::MetaZoneGenericNamesLongV1Marker>
+            + DataProvider<'data, provider::time_zones::MetaZoneGenericNamesShortV1Marker>
+            + DataProvider<'data, provider::time_zones::MetaZoneSpecificNamesLongV1Marker>
+            + DataProvider<'data, provider::time_zones::MetaZoneSpecificNamesShortV1Marker>
             + ?Sized,
     {
         let locale = locale.into();
 
-        let zone_formats: Cow<TimeZoneFormatsV1> = zone_provider
+        let zone_formats: DataPayload<TimeZoneFormatsV1Marker> = zone_provider
             .load_payload(&DataRequest {
                 resource_path: ResourcePath {
                     key: provider::key::TIMEZONE_FORMATS_V1,
@@ -156,8 +159,7 @@ impl<'d> TimeZoneFormat<'d> {
                     },
                 },
             })?
-            .payload
-            .take()?;
+            .take_payload()?;
 
         let mut time_zone_format = Self {
             pattern,
@@ -294,7 +296,7 @@ impl<'d> TimeZoneFormat<'d> {
     // TODO(#622) Make this public once TimeZoneFormat is public.
     //           And remove #[allow(unused)]
     #[allow(unused)]
-    pub(super) fn format<'l: 'd, T>(&'l self, value: &'l T) -> FormattedTimeZone<'l, T>
+    pub(super) fn format<'l: 'data, T>(&'l self, value: &'l T) -> FormattedTimeZone<'l, T>
     where
         T: TimeZoneInput,
     {
@@ -343,10 +345,10 @@ impl<'d> TimeZoneFormat<'d> {
     #[allow(unused)]
     pub(super) fn format_to_write(
         &self,
-        w: &mut impl std::fmt::Write,
+        w: &mut impl core::fmt::Write,
         value: &impl TimeZoneInput,
     ) -> fmt::Result {
-        time_zone::write_pattern(self, value, w).map_err(|_| std::fmt::Error)
+        time_zone::write_pattern(self, value, w).map_err(|_| core::fmt::Error)
     }
 
     /// Takes a [`TimeZoneInput`] implementer and returns a string with the formatted value.
@@ -401,8 +403,14 @@ impl<'d> TimeZoneFormat<'d> {
             &self
                 .exemplar_cities
                 .as_ref()
+                .map(|p| p.get())
                 .and_then(|cities| time_zone.time_zone_id().and_then(|id| cities.get(id)))
-                .map(|location| self.zone_formats.region_format.replace("{0}", location))
+                .map(|location| {
+                    self.zone_formats
+                        .get()
+                        .region_format
+                        .replace("{0}", location)
+                })
                 .ok_or(fmt::Error)?,
         )
         .map_err(DateTimeFormatError::from)
@@ -419,6 +427,7 @@ impl<'d> TimeZoneFormat<'d> {
         sink.write_str(
             self.mz_generic_short
                 .as_ref()
+                .map(|p| p.get())
                 .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
                 .ok_or(fmt::Error)?,
         )
@@ -436,6 +445,7 @@ impl<'d> TimeZoneFormat<'d> {
         sink.write_str(
             self.mz_generic_long
                 .as_ref()
+                .map(|p| p.get())
                 .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
                 .ok_or(fmt::Error)?,
         )
@@ -453,6 +463,7 @@ impl<'d> TimeZoneFormat<'d> {
         sink.write_str(
             self.mz_specific_short
                 .as_ref()
+                .map(|p| p.get())
                 .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
                 .and_then(|specific_names| {
                     time_zone
@@ -475,6 +486,7 @@ impl<'d> TimeZoneFormat<'d> {
         sink.write_str(
             self.mz_specific_long
                 .as_ref()
+                .map(|p| p.get())
                 .and_then(|metazones| time_zone.metazone_id().and_then(|mz| metazones.get(mz)))
                 .and_then(|specific_names| {
                     time_zone
@@ -499,20 +511,21 @@ impl<'d> TimeZoneFormat<'d> {
     ) -> Result<(), DateTimeFormatError> {
         let gmt_offset = time_zone.gmt_offset();
         if gmt_offset.is_zero() {
-            sink.write_str(&self.zone_formats.gmt_zero_format.clone())
+            sink.write_str(&self.zone_formats.get().gmt_zero_format.clone())
                 .map_err(DateTimeFormatError::from)
         } else {
             // TODO(blocked on #277) Use formatter utility instead of replacing "{0}".
             sink.write_str(
                 &self
                     .zone_formats
+                    .get()
                     .gmt_format
                     .replace(
                         "{0}",
                         if gmt_offset.is_positive() {
-                            &self.zone_formats.hour_format.0
+                            &self.zone_formats.get().hour_format.0
                         } else {
-                            &self.zone_formats.hour_format.1
+                            &self.zone_formats.get().hour_format.1
                         },
                     )
                     // support all combos of "(HH|H):mm" by replacing longest patterns first.
@@ -533,6 +546,7 @@ impl<'d> TimeZoneFormat<'d> {
         sink.write_str(
             self.exemplar_cities
                 .as_ref()
+                .map(|p| p.get())
                 .and_then(|cities| time_zone.time_zone_id().and_then(|id| cities.get(id)))
                 .ok_or(fmt::Error)?,
         )
@@ -553,6 +567,7 @@ impl<'d> TimeZoneFormat<'d> {
         sink.write_str(
             self.exemplar_cities
                 .as_ref()
+                .map(|p| p.get())
                 .and_then(|cities| cities.get("Etc/Unknown"))
                 .unwrap_or(&Cow::Borrowed("Unknown")),
         )
